@@ -6,7 +6,7 @@ DOCSIS::ConfigFile - Decodes and encodes DOCSIS config-files
 
 =head1 VERSION
 
-0.60
+0.6002
 
 =head1 SYNOPSIS
 
@@ -81,6 +81,7 @@ values are still safe to do, but L</encode> and L</decode> might die.
 use strict;
 use warnings;
 use autodie;
+use Carp qw/ carp confess /;
 use Digest::MD5;
 use Digest::HMAC_MD5;
 use DOCSIS::ConfigFile::Syminfo;
@@ -91,14 +92,41 @@ use constant Syminfo => "DOCSIS::ConfigFile::Syminfo";
 use constant Decode  => "DOCSIS::ConfigFile::Decode";
 use constant Encode  => "DOCSIS::ConfigFile::Encode";
 
-our $VERSION = '0.60';
+our $VERSION = '0.6002';
 our $TRACE   = 0;
 
+=head1 ATTRIBUTES
+
+=head2 shared_secret
+
+Sets or gets the shared secret.
+
+=cut
+
+sub shared_secret {
+    my $self = shift;
+    $self->{'shared_secret'} = $_[0] if(@_);
+    return $self->{'shared_secret'} ||= q();
+}
+
+=head2 advanced_output
+
+Sets weither advanced output should be enabled. Takes 0 or 1 as argument.
+Advanced output is off (0) by default.
+
+=cut
+
+sub advanced_output {
+    my $self = shift;
+    $self->{'advanced_output'} = $_[0] if(@_);
+    return $self->{'advanced_output'} || 0;
+}
+
 =head1 METHODS
 
-=head2 new(%args)
+=head2 new
 
-Object constructor.
+    $self = $class->new(\%args);
 
 Arguments can be:
 
@@ -117,11 +145,13 @@ sub new {
 
 =head2 decode
 
-Decodes a binary config-file. Needs only one of these arguments: Filehandle,
-path to file or reference to a binary string.
+    $array_ref = $self->decode($path_to_file);
+    $array_ref = $self->decode(\$binary_string);
+    $array_ref = $self->decode($FH);
 
-Returns an array-ref of hashes, containing the config as a perl data
-structure.
+This method decodes a binary config file stored in either a file on disk,
+a binary string, or a filehandle. It returns an array-ref of hashes,
+containing the config as a perl data structure.
 
 =cut
 
@@ -257,10 +287,12 @@ sub _value_to_cfg {
 
 =head2 encode
 
-Encodes an array of hashes, containing the DOCSIS config-file settings. Takes
-only on argument: An array-ref of hashes.
+    $binary_str = $self->encode([ { ... }, ... ]);
 
-Returns a binary string.
+Encodes an array of hashes, containing the DOCSIS config-file settings and
+returns a binary encoded string. See L</DESCRIPTION> and the unit tests for
+example input. For other structures, see the table generated by
+L<DOCSIS::ConfigFile::Syminfo/dump_symbol_tree>.
 
 =cut
 
@@ -304,17 +336,12 @@ sub _encode_loop {
 
     TLV:
     for my $tlv (@$config) {
+        confess sprintf 'Invalid TLV#%s: %s', $i, $tlv || '__undefined_tlv__' unless(ref $tlv eq 'HASH');
+        confess sprintf 'Missing name in TLV#%s: %s', $i, join(',', keys %$tlv) unless($tlv->{'name'});
 
-        unless(ref $tlv eq 'HASH') {
-            confess sprintf 'Invalid TLV#%s: %s', $i, $tlv || '__undefined_tlv__';
-        }
-        unless($tlv->{'name'}) {
-            confess sprintf 'Missing name in TLV#%s: %s', $i, join(',', keys %$tlv);
-        }
-
-        my $name    = $tlv->{'name'};
+        my $name = $tlv->{'name'};
         my $syminfo = Syminfo->from_id($name);
-        my($type, $length, $value, @error);
+        my($type, $length, $value);
 
         unless($syminfo->func) {
             carp sprintf 'Unknown encode method for TLV#%s/%s', $i, $name;
@@ -333,9 +360,7 @@ sub _encode_loop {
             }
             unless(defined $tlv->{'value'}) {
                 confess sprintf 'Missing value in TLV#%s/%s', $i, $name;
-                next TLV;
             }
-
             unless(defined( $value = $sub->($tlv) )) {
                 carp sprintf 'Undefined encoded value for TLV#%s/%s', $i, $name;
                 next TLV;
@@ -344,36 +369,13 @@ sub _encode_loop {
             $value = pack 'C*', @$value;
         }
 
-        SIBLING:
-        for my $o (@{ $syminfo->siblings }) {
-            next unless($o->l_limit or $o->u_limit);
-
-            my $length = $value =~ /^\d+$/ ? $value : length $value;
-
-            if($length > $o->u_limit) {
-                push @error, sprintf '%s/%s: %s > %s', $o->pcode, $o->code, $length, $o->u_limit;
-            }
-            elsif($length < $o->l_limit) {
-                push @error, sprintf '%s/%s: %s < %s', $o->pcode, $o->code, $length, $o->l_limit;
-            }
-            else {
-                $syminfo = $o;
-                @error   = ();
-                last SIBLING;
-            }
-        }
-
-        if(@error) {
-            confess sprintf 'Invalid value for %s: %s', $name, join(', ', @error);
-        }
-
-        $type   = $syminfo->code;
-        $length = ($syminfo->length == 2) ? pack("n", length $value)
-                :                           pack("C", length $value);
+        $syminfo = $self->_syminfo_from_syminfo_siblings($syminfo, \$value);
+        $type = $syminfo->code;
+        $length = ($syminfo->length == 2) ? pack('n', length $value) : pack('C', length $value);
 
         #carp 'name=%s type=%i, length=%i', $name, $type, length($value);
 
-        $type       = pack "C", $type;
+        $type = pack "C", $type;
         $binstring .= "$type$length$value";
 
         $self->_calculate_cmts_mic($name, "$type$length$value");
@@ -385,6 +387,33 @@ sub _encode_loop {
     return $binstring;
 }
 
+sub _syminfo_from_syminfo_siblings {
+    my($self, $syminfo, $value) = @_;
+    my @error;
+
+    SIBLING:
+    for my $sibling (@{ $syminfo->siblings }) {
+        unless($sibling->l_limit or $sibling->u_limit) {
+            next SIBLING;
+        }
+
+        my $length = $$value =~ /^\d+$/ ? $$value : length $$value;
+
+        if($length > $sibling->u_limit) {
+            push @error, sprintf '%s/%s: %s > %s', $sibling->pcode, $sibling->code, $length, $sibling->u_limit;
+        }
+        elsif($length < $sibling->l_limit) {
+            push @error, sprintf '%s/%s: %s < %s', $sibling->pcode, $sibling->code, $length, $sibling->l_limit;
+        }
+        else {
+            return $sibling;
+        }
+    }
+
+    confess sprintf 'Invalid value for %s: %s', $syminfo->id, join(', ', @error) if(@error);
+    return $syminfo;
+}
+
 sub _calculate_eod_and_pad {
     my $self   = shift;
     my $length = length $self->{'binstring'};
@@ -424,31 +453,6 @@ sub _calculate_cmts_mic {
     }
 }
 
-=head2 shared_secret
-
-Sets or gets the shared secret.
-
-=cut
-
-sub shared_secret {
-    my $self = shift;
-    $self->{'shared_secret'} = shift if(@_);
-    return $self->{'shared_secret'} ||= q();
-}
-
-=head2 advanced_output
-
-Sets weither advanced output should be enabled. Takes 0 or 1 as argument.
-Advanced output is off (0) by default.
-
-=cut
-
-sub advanced_output {
-    my $self = shift;
-    $self->{'advanced_output'} = $_[0] if(@_);
-    return $self->{'advanced_output'} || 0;
-}
-
 =head1 CONSTANTS
 
 =head2 Decode
